{% load staticfiles %}

<script src="{% static 'node_modules/clipboard/dist/clipboard.min.js' %}"></script>
<script>
    $( document ).ready(function() {
        new ClipboardJS('.cpy');
    });
</script>
