{% extends base_template|default:"handyhelpers/handyhelpers_base.htm" %}
{% load staticfiles %}

{% block local_head %}
    <script src="{% static 'node_modules/chart.js/dist/Chart.bundle.min.js' %}"></script>
{% endblock local_head %}

{% block content %}
    {% include 'handyhelpers/report/annual_trends_content.htm' %}
{% endblock content %}
